#!/usr/bin/perl -w
#
# WardenStatus.pm
#
# Copyright (C) 2011 Cesnet z.s.p.o
# Author(s): 	Tomas PLESNIK 	<plesnik@ics.muni.cz>
#		Jan SOUKAL	<soukal@ics.muni.cz>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Cesnet z.s.p.o nor the names of its
#    contributors may be used to endorse or promote products derived from
#     this software without specific prior written permission.
#
# This software is provided ``as is'', and any express or implied
# warranties, including, but not limited to, the implied warranties of
# merchantability and fitness for a particular purpose are disclaimed.
# In no event shall the Cesnet z.s.p.o or contributors be liable for
# any direct, indirect, incidental, special, exemplary, or consequential
# damages (including, but not limited to, procurement of substitute
# goods or services; loss of use, data, or profits; or business
# interruption) however caused and on any theory of liability, whether
# in contract, strict liability, or tort (including negligence or
# otherwise) arising in any way out of the use of this software, even
# if advised of the possibility of such damage.

package WardenStatus;

use strict;
use SOAP::Lite;
use IO::Socket::SSL qw(debug1);
use SOAP::Transport::TCP

our $VERSION = 100;

#-------------------------------------------------------------------------------
# errMsg - print error message and die
#-------------------------------------------------------------------------------
sub errMsg
{
  my $msg = shift;
  die($msg . "\n");
} # End of errMsg


#-------------------------------------------------------------------------------
# c2s - connect to server, send request and receive response
#-------------------------------------------------------------------------------
sub c2s 
{
  my $uri		= shift;
  my $ssl_key_file	= shift;
  my $ssl_cert_file	= shift;
  my $ssl_ca_file	= shift;
  my $method		= shift;

  my $client;
  my ($server, $port, $service) = $uri =~ /https:\/\/(.+)\:(\d+)\/(.+)/;
  if (!($client = SOAP::Transport::TCP::Client->new(
    PeerAddr            => $server,
    PeerPort            => $port,
    Proto               => 'tcp',
    SSL_use_cert        => 1,
    SSL_verify_mode     => 0x02,
    SSL_key_file        => $ssl_key_file,
    SSL_cert_file       => $ssl_cert_file,
    SSL_ca_file         => $ssl_ca_file,
  ))) {errMsg("Sorry, unable to create socket: " . &SOAP::Transport::TCP::Client::errstr)}

  # setting of URI and serialize SOAP envelope and data object
  my $soap     = SOAP::Lite->uri($uri);
  my $envelope = $soap->serializer->envelope(method => $method);

  # setting of TCP URI and send serialized SOAP envelope and data
  my $tcp_uri = "tcp://$server:$port/$service";
  my $result = $client->send_receive(envelope => $envelope, endpoint => $tcp_uri);

  # check server response
  if (!defined $result) {
    errMsg("Error: server returned empty response. Probably problem with used SSL ceritificates.");
  } else {
    # deserialized response from server -> create SOAP envelope and data object
    my $response = $soap->deserializer->deserialize($result);
    # check SOAP fault status
    $response->fault ? errMsg("Server sent error message:: " . $response->faultstring) : return $response;
  }
}


#-------------------------------------------------------------------------------
# getClients - get list of registered clients
#-------------------------------------------------------------------------------
sub getClients
{
  my $warden_path = shift;

  my $etcdir = $warden_path . "/etc/";
  my $libdir = $warden_path . "/lib/";

  # read the config file
  require $libdir .  "WardenConf.pm";
  my $conf_file = $etcdir . "warden-client.conf";
  my ($uri, $ssl_key_file, $ssl_cert_file, $ssl_ca_file) = WardenConf::loadConf($conf_file);

  my $response = c2s($uri, $ssl_key_file, $ssl_cert_file, $ssl_ca_file, "getClients");
  # match getClients functions response
  $response->match('/Envelope/Body/getClientsResponse/');

  my $i = 1;
  my ($client_id, $hostname, $registered, $requestor, $service, $client_type, $type, $receive_own_events, $description_tags, $ip_net_client);
  my @clients;
  my $response_data = $response->valueof("[$i]");

  # parse returned data object
  while (defined $response_data) {
    # inicialization of temporal client array
    my @client;

    # parse items of one client
    $client_id		= $response_data->{'CLIENT_ID'};
    $hostname		= $response_data->{'HOSTNAME'};
    $registered		= $response_data->{'REGISTERED'};
    $requestor 		= $response_data->{'REQUESTOR'};
    $service	 	= $response_data->{'SERVICE'};
    $client_type	= $response_data->{'CLIENT_TYPE'};
    $type		= $response_data->{'TYPE'};
    $receive_own_events	= $response_data->{'RECEIVE_OWN_EVENTS'};
    $description_tags	= $response_data->{'DESCRIPTION_TAGS'};
    $ip_net_client	= $response_data->{'IP_NET_CLIENT'};

    # push received clients from warden server into @clients which is returned
    @client = ("$client_id", "$hostname", "$registered", "$requestor", "$service", "$client_type", "$type", "$receive_own_events", "$description_tags", "$ip_net_client");
    push (@clients,\@client);

    # go to the next received client
    $i++;
    $response_data = $response->valueof("[$i]");
  }
  return @clients;
} # End of getClients


#-------------------------------------------------------------------------------
# getStatus - get warden server status
#-------------------------------------------------------------------------------
sub getStatus
{
  my $warden_path = shift;

  my $etcdir = $warden_path . "/etc/";
  my $libdir = $warden_path . "/lib/";

  # read the config file
  require $libdir .  "WardenConf.pm";
  my $conf_file = $etcdir . "warden-client.conf";
  my ($uri, $ssl_key_file, $ssl_cert_file, $ssl_ca_file) = WardenConf::loadConf($conf_file);

  my $response = c2s($uri, $ssl_key_file, $ssl_cert_file, $ssl_ca_file, "getStatus");

  # match getStatus functions response
  $response->match('/Envelope/Body/getStatusResponse/');
  my @status;
  my $i = 1;

  # get first value from SOAP data object
  my $response_data = $response->valueof("[$i]");
  # parse items of server status
  my $db_size			= $response_data->{'DB_SIZE'};
  my $events_sum		= $response_data->{'EVENTS_SUM'};
  my $events_last_id		= $response_data->{'EVENTS_LAST_ID'};
  my $events_first_timestamp	= $response_data->{'EVENTS_FIRST_TIMESTAMP'};
  my $events_last_timestamp	= $response_data->{'EVENTS_LAST_TIMESTAMP'};
  my $clients_sum	 	= $response_data->{'CLIENTS_SUM'};
  my @server_status = ("$db_size", "$events_sum", "$events_last_id", "$events_first_timestamp", "$events_last_timestamp", "$clients_sum");
  push(@status, \@server_status);

  $i++;
  $response_data = $response->valueof("[$i]");
  while (defined $response_data) {
    my @client_status;
    # parse SOAP data object
    my $client_id	= $response_data->{'CLIENT_ID'};
    my $hostname	= $response_data->{'HOSTNAME'};
    my $service 	= $response_data->{'SERVICE'};
    my $count		= $response_data->{'COUNT'};
    my $timestamp	= $response_data->{'TIMESTAMP'};
    @client_status = ("$client_id", "$hostname", "$service", "$count", "$timestamp");
    push(@status, \@client_status);
    $i++;
    $response_data = $response->valueof("[$i]");
  }

  return @status;
} # End of getStatus

1;
